@@ -1,4 +1,6 @@
 use std::io;
+use std::iter;
+use time::strftime;
 use lazysort::SortedBy;
 use std::path::PathBuf;
 use std::fs::{self, File};
@@ -6,8 +8,8 @@ use iron::modifiers::Header;
 use self::super::{Options, Error};
 use mime_guess::guess_mime_type_opt;
 use iron::{headers, status, method, mime, IronResult, Listening, Response, TypeMap, Request, Handler, Iron};
-use self::super::util::{url_path, is_symlink, html_response, file_contains, percent_decode, detect_file_as_dir, file_time_modified, USER_AGENT, ERROR_HTML,
-                        DIRECTORY_LISTING_HTML};
+use self::super::util::{url_path, is_symlink, html_response, file_binary, percent_decode, detect_file_as_dir, file_time_modified, human_readable_size,
+                        USER_AGENT, ERROR_HTML, INDEX_EXTENSIONS, DIRECTORY_LISTING_HTML};
 
 
 #[derive(Clone, Hash, Ord, PartialOrd, Eq, PartialEq)]
@@ -15,6 +17,7 @@ pub struct HttpHandler {
     pub hosted_directory: (String, PathBuf),
     pub follow_symlinks: bool,
     pub temp_directory: Option<(String, PathBuf)>,
+    pub check_indices: bool,
 }
 
 impl HttpHandler {
@@ -23,6 +26,7 @@ impl HttpHandler {
             hosted_directory: opts.hosted_directory.clone(),
             follow_symlinks: opts.follow_symlinks,
             temp_directory: opts.temp_directory.clone(),
+            check_indices: opts.check_indices,
         }
     }
 }
@@ -91,7 +95,7 @@ impl HttpHandler {
     }
 
     fn handle_get_file(&self, req: &mut Request, req_p: PathBuf) -> IronResult<Response> {
-        let mime_type = guess_mime_type_opt(&req_p).unwrap_or_else(|| if file_contains(&req_p, 0) {
+        let mime_type = guess_mime_type_opt(&req_p).unwrap_or_else(|| if file_binary(&req_p) {
             "application/octet-stream".parse().unwrap()
         } else {
             "text/plain".parse().unwrap()
@@ -105,6 +109,42 @@ impl HttpHandler {
     }
 
     fn handle_get_dir(&self, req: &mut Request, req_p: PathBuf) -> IronResult<Response> {
+        if self.check_indices {
+            let mut idx = req_p.join("index");
+            if let Some(e) = INDEX_EXTENSIONS.iter()
+                .find(|e| {
+                    idx.set_extension(e);
+                    idx.exists()
+                }) {
+                if req.url.path().pop() == Some("") {
+                    let r = self.handle_get_file(req, idx);
+                    println!("{} found index file for directory {}",
+                             iter::repeat(' ').take(req.remote_addr.to_string().len()).collect::<String>(),
+                             req_p.display());
+                    return r;
+                } else {
+                    return self.handle_get_dir_index_no_slash(req, e);
+                }
+            }
+        }
+
+        self.handle_get_dir_listing(req, req_p)
+    }
+
+    fn handle_get_dir_index_no_slash(&self, req: &mut Request, idx_ext: &str) -> IronResult<Response> {
+        let new_url = req.url.to_string() + "/";
+        println!("Redirecting {} to {} - found index file index.{}", req.remote_addr, new_url, idx_ext);
+
+        // We redirect here because if we don't and serve the index right away funky shit happens.
+        // Example:
+        //   - Without following slash:
+        //     https://cloud.githubusercontent.com/assets/6709544/21442017/9eb20d64-c89b-11e6-8c7b-888b5f70a403.png
+        //   - With following slash:
+        //     https://cloud.githubusercontent.com/assets/6709544/21442028/a50918c4-c89b-11e6-8936-c29896947f6a.png
+        Ok(Response::with((status::MovedPermanently, Header(headers::Server(USER_AGENT.to_string())), Header(headers::Location(new_url)))))
+    }
+
+    fn handle_get_dir_listing(&self, req: &mut Request, req_p: PathBuf) -> IronResult<Response> {
         let relpath = (url_path(&req.url) + "/").replace("//", "/");
         println!("{} was served directory listing for {}", req.remote_addr, req_p.display());
         Ok(Response::with((status::Ok,
@@ -112,6 +152,18 @@ impl HttpHandler {
                            "text/html;charset=utf-8".parse::<mime::Mime>().unwrap(),
                            html_response(DIRECTORY_LISTING_HTML,
                                          &[&relpath,
+                                           &if self.temp_directory.is_some() {
+                                               r#"<script type="text/javascript">{drag_drop}</script>"#.to_string()
+                                           } else {
+                                               String::new()
+                                           },
+                                           &if &req.url.path() == &[""] {
+                                               String::new()
+                                           } else {
+                                               format!("<tr><td><a href=\"../\"><img id=\"parent_dir\" src=\"{{back_arrow_icon}}\"></img></a></td> \
+                                                        <td><a href=\"../\">Parent directory</a></td> <td>{}</td> <td></td></tr>",
+                                                       strftime("%F %T", &file_time_modified(req_p.parent().unwrap())).unwrap())
+                                           },
                                            &req_p.read_dir()
                                                .unwrap()
                                                .map(Result::unwrap)
@@ -121,13 +173,47 @@ impl HttpHandler {
                                                        .cmp(&(rhs.file_type().unwrap().is_file(), rhs.file_name().to_str().unwrap().to_lowercase()))
                                                })
                                                .fold("".to_string(), |cur, f| {
-                let fname = f.file_name().into_string().unwrap() +
-                            if !f.file_type().unwrap().is_file() {
-                    "/"
+                let url = format!("/{}", relpath).replace("//", "/");
+                let is_file = f.file_type().unwrap().is_file();
+                let path = f.path();
+                let fname = f.file_name().into_string().unwrap();
+                let len = f.metadata().unwrap().len();
+                let mime = if is_file {
+                    match guess_mime_type_opt(&path) {
+                        Some(mime::Mime(mime::TopLevel::Image, ..)) |
+                        Some(mime::Mime(mime::TopLevel::Video, ..)) => "_image",
+                        Some(mime::Mime(mime::TopLevel::Text, ..)) => "_text",
+                        Some(mime::Mime(mime::TopLevel::Application, ..)) => "_binary",
+                        None => if file_binary(&path) { "" } else { "_text" },
+                        _ => "",
+                    }
                 } else {
                     ""
                 };
-                cur + "<li><a href=\"" + &format!("/{}", relpath).replace("//", "/") + &fname + "\">" + &fname + "</a></li>\n"
+
+                format!("{}<tr><td><a href=\"{}{}\"><img id=\"{}\" src=\"{{{}{}_icon}}\"></img></a></td> <td><a href=\"{}{}\">{}{}</a></td> <td>{}</td> \
+                         <td><abbr title=\"{} B\">{}</abbr></td></tr>\n",
+                        cur,
+                        url,
+                        fname,
+                        path.file_stem().map(|p| p.to_str().unwrap()).unwrap_or(&fname),
+                        if is_file { "file" } else { "dir" },
+                        mime,
+                        url,
+                        fname,
+                        fname,
+                        if is_file { "" } else { "/" },
+                        strftime("%F %T", &file_time_modified(&path)).unwrap(),
+                        if is_file {
+                            len.to_string()
+                        } else {
+                            String::new()
+                        },
+                        if is_file {
+                            human_readable_size(len)
+                        } else {
+                            String::new()
+                        })
             })]))))
     }
 
