@@ -11,7 +11,7 @@
 //! ```
 
 
-use clap::{AppSettings, App, Arg};
+use clap::{AppSettings, Arg};
 use std::path::PathBuf;
 use std::env::temp_dir;
 use std::str::FromStr;
@@ -29,16 +29,15 @@ pub struct Options {
     pub follow_symlinks: bool,
     /// The temp directory to write to before copying to hosted directory. Default: `None`
     pub temp_directory: Option<(String, PathBuf)>,
+    /// Whether to check for index files in served directories before serving a listing. Default: true
+    pub check_indices: bool,
 }
 
 impl Options {
     /// Parse `env`-wide command-line arguments into an `Options` instance
     pub fn parse() -> Options {
-        let matches = App::new("http")
-            .version(crate_version!())
-            .author(crate_authors!())
+        let matches = app_from_crate!()
             .setting(AppSettings::ColoredHelp)
-            .about("Host These Things Please - a basic HTTP server for hosting a folder fast and simply")
             .arg(Arg::from_usage("[DIR] 'Directory to host. Default: current working directory'")
                 .validator(|s| Options::filesystem_dir_validator(s, "Directory to host")))
             .arg(Arg::from_usage("-p --port [port] 'Port to use. Default: first free port from 8000 up'").validator(Options::u16_validator))
@@ -46,6 +45,7 @@ impl Options {
                 .validator(|s| Options::filesystem_dir_validator(s, "Temporary directory")))
             .arg(Arg::from_usage("-s --follow-symlinks 'Follow symlinks. Default: false'"))
             .arg(Arg::from_usage("-w --allow-write 'Allow for write operations. Default: false'"))
+            .arg(Arg::from_usage("-i --no-indices 'Always generate dir listings even if index files are available. Default: false'"))
             .get_matches();
 
         let dir = matches.value_of("DIR").unwrap_or(".");
@@ -75,6 +75,7 @@ impl Options {
             } else {
                 None
             },
+            check_indices: !matches.is_present("no-indices"),
         }
     }
 
