@@ -1,13 +1,18 @@
 //! Module containing various utility functions.
 
 
+use base64;
+use std::f64;
+use std::cmp;
 use iron::Url;
 use std::fs::File;
-use std::io::Read;
 use std::path::Path;
 use std::borrow::Cow;
 use url::percent_encoding;
+use std::collections::HashMap;
 use time::{self, Duration, Tm};
+use std::io::{BufReader, BufRead};
+use mime_guess::get_mime_type_str;
 
 
 /// The generic HTML page to use as response to errors.
@@ -16,6 +21,40 @@ pub static ERROR_HTML: &'static str = include_str!("../assets/error.html");
 /// The HTML page to use as template for a requested directory's listing.
 pub static DIRECTORY_LISTING_HTML: &'static str = include_str!("../assets/directory_listing.html");
 
+lazy_static! {
+    pub static ref ASSETS: HashMap<&'static str, Cow<'static, str>> = {
+        let mut ass = HashMap::with_capacity(1);
+        ass.insert("favicon",
+            Cow::Owned(format!("data:{};base64,{}", get_mime_type_str("ico").unwrap(), base64::encode(include_bytes!("../assets/favicon.ico")))));
+        ass.insert("dir_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               base64::encode(include_bytes!("../assets/icons/directory_icon.gif")))));
+        ass.insert("file_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               base64::encode(include_bytes!("../assets/icons/file_icon.gif")))));
+        ass.insert("file_binary_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               base64::encode(include_bytes!("../assets/icons/file_binary_icon.gif")))));
+        ass.insert("file_image_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               base64::encode(include_bytes!("../assets/icons/file_image_icon.gif")))));
+        ass.insert("file_text_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               base64::encode(include_bytes!("../assets/icons/file_text_icon.gif")))));
+        ass.insert("back_arrow_icon",
+            Cow::Owned(format!("data:{};base64,{}",
+                               get_mime_type_str("gif").unwrap(),
+                               base64::encode(include_bytes!("../assets/icons/back_arrow_icon.gif")))));
+        ass.insert("drag_drop", Cow::Borrowed(include_str!("../assets/drag_drop.js")));
+        ass
+    };
+}
+
 /// The port to start scanning from if no ports were given.
 pub static PORT_SCAN_LOWEST: u16 = 8000;
 
@@ -25,6 +64,9 @@ pub static PORT_SCAN_HIGHEST: u16 = 9999;
 /// The app name and version to use with User-Agent or Server response header.
 pub static USER_AGENT: &'static str = concat!("http/", env!("CARGO_PKG_VERSION"));
 
+/// Index file extensions to look for if `-i` was not specified.
+pub static INDEX_EXTENSIONS: &'static [&'static str] = &["html", "htm", "shtml"];
+
 
 /// Uppercase the first character of the supplied string.
 ///
@@ -44,34 +86,22 @@ pub fn uppercase_first(s: &str) -> String {
     }
 }
 
-/// Check if the specified file contains the specified byte.
+/// Check if the specified file is to be considered "binary".
+///
+/// Basically checks is a file is UTF-8.
 ///
 /// # Examples
 ///
 /// ```
-/// # use https::util::file_contains;
+/// # use https::util::file_binary;
 /// # #[cfg(target_os = "windows")]
-/// # assert!(file_contains("target/debug/http.exe", 0));
+/// # assert!(file_binary("target/debug/http.exe"));
 /// # #[cfg(not(target_os = "windows"))]
-/// assert!(file_contains("target/debug/http", 0));
-/// assert!(!file_contains("Cargo.toml", 0));
+/// assert!(file_binary("target/debug/http"));
+/// assert!(!file_binary("Cargo.toml"));
 /// ```
-pub fn file_contains<P: AsRef<Path>>(path: P, byte: u8) -> bool {
-    if let Ok(mut f) = File::open(path) {
-        let mut buf = [0u8; 1024];
-
-        while let Ok(read) = f.read(&mut buf) {
-            if buf[..read].contains(&byte) {
-                return true;
-            }
-
-            if read < buf.len() {
-                break;
-            }
-        }
-    }
-
-    false
+pub fn file_binary<P: AsRef<Path>>(path: P) -> bool {
+    File::open(path).and_then(|f| BufReader::new(f).read_line(&mut String::new())).is_err()
 }
 
 /// Fill out an HTML template.
@@ -85,7 +115,8 @@ pub fn file_contains<P: AsRef<Path>>(path: P, byte: u8) -> bool {
 /// println!(html_response(NOT_IMPLEMENTED_HTML, &["<p>Abolish the burgeoisie!</p>"]));
 /// ```
 pub fn html_response<S: AsRef<str>>(data: &str, format_strings: &[S]) -> String {
-    format_strings.iter().enumerate().fold(data.to_string(), |d, (i, s)| d.replace(&format!("{{{}}}", i), s.as_ref()))
+    ASSETS.iter().fold(format_strings.iter().enumerate().fold(data.to_string(), |d, (i, s)| d.replace(&format!("{{{}}}", i), s.as_ref())),
+                       |d, (k, v)| d.replace(&format!("{{{}}}", k), v))
 }
 
 /// Return the path part of the URL.
@@ -101,9 +132,14 @@ pub fn html_response<S: AsRef<str>>(data: &str, format_strings: &[S]) -> String
 /// assert_eq!(url_path(&url), "capitalism/русский/");
 /// ```
 pub fn url_path(url: &Url) -> String {
-    url.path().into_iter().fold("".to_string(),
-                                |cur, pp| format!("{}/{}", cur, percent_decode(pp).unwrap_or(Cow::Borrowed("<incorrect UTF8>"))))[1..]
-        .to_string()
+    let path = url.path();
+    if path == [""] {
+        "/".to_string()
+    } else {
+        path.into_iter().fold("".to_string(),
+                              |cur, pp| format!("{}/{}", cur, percent_decode(pp).unwrap_or(Cow::Borrowed("<incorrect UTF8>"))))[1..]
+            .to_string()
+    }
 }
 
 /// Decode a percent-encoded string (like a part of a URL).
@@ -157,3 +193,24 @@ pub fn detect_file_as_dir(mut p: &Path) -> bool {
 pub fn is_symlink<P: AsRef<Path>>(p: P) -> bool {
     p.as_ref().read_link().is_ok()
 }
+
+/// Construct string representing a human-readable size.
+///
+/// Stolen, adapted and inlined from [fielsize.js](http://filesizejs.com).
+pub fn human_readable_size(s: u64) -> String {
+    if s == 0 {
+        "0 B".to_string()
+    } else {
+        let num = s as f64;
+        let exp = cmp::min(cmp::max((num.log(f64::consts::E) / 1024f64.log(f64::consts::E)) as i32, 0), 8);
+
+        let val = num / 2f64.powi(exp * 10);
+
+        if exp > 0 {
+                (val * 10f64).round() / 10f64
+            } else {
+                val.round()
+            }
+            .to_string() + " " + ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"][cmp::max(exp, 0) as usize]
+    }
+}
